@@ -36,20 +36,29 @@
 #include <string.h>
 #include <sys/mman.h>
 #include <unistd.h>
+#include <sys/param.h>
 
+#include "config.h"
 #include "ld_rtld.h"
 #include "msg_queue.h"
 #include "ulp.h"
 #include "ulp_common.h"
+#include "arch_common.h"
 
 /* This header should be included last, as it poisons some symbols.  */
 #include "error.h"
 
 /* Declare internal glibc functions which are exposed. We have to use them
    while __ulp_asunsafe_begin have not run yet.  */
+extern void __libc_free(void *);
 extern void *__libc_malloc(size_t);
 extern void *__libc_calloc(size_t, size_t);
-extern void __libc_free(void *);
+extern void *__libc_realloc(void *, size_t);
+extern void *__libc_valloc(size_t);
+extern void *__libc_pvalloc(size_t);
+extern void *__libc_memalign(size_t, size_t);
+/* aligned_alloc doesn't have an alternate public symbol.  */
+/* posix_memalign doesn't have an alternate public symbol.  */
 extern const char *__progname;
 
 static int flag = 0;
@@ -59,7 +68,6 @@ static void (*real_free)(void *) = NULL;
 static void *(*real_malloc)(size_t) = NULL;
 static void *(*real_calloc)(size_t, size_t) = NULL;
 static void *(*real_realloc)(void *, size_t) = NULL;
-static void *(*real_reallocarray)(void *, size_t, size_t) = NULL;
 static void *(*real_valloc)(size_t) = NULL;
 static void *(*real_pvalloc)(size_t) = NULL;
 static void *(*real_memalign)(size_t, size_t) = NULL;
@@ -393,8 +401,7 @@ get_ld_global_locks()
   const char *tok;
   int major, minor;
 
-  void *rtld_global =
-      get_loaded_symbol_addr("ld-linux-x86-64.so.2", "_rtld_global", NULL);
+  void *rtld_global = get_loaded_symbol_addr(LD_LINUX, "_rtld_global", NULL);
   if (!rtld_global) {
     libpulp_crash("symbol _rtld_global not found in ld-linux-x86_64.so\n");
   }
@@ -591,6 +598,8 @@ __ulp_asunsafe_begin(void)
   if (real_malloc)
     return;
 
+  bool ok = true;
+
   real_dlopen = dlsym(RTLD_NEXT, "dlopen");
   real_dlmopen = dlsym(RTLD_NEXT, "dlmopen");
   real_dlclose = dlsym(RTLD_NEXT, "dlclose");
@@ -598,35 +607,90 @@ __ulp_asunsafe_begin(void)
   real_dladdr1 = dlsym(RTLD_NEXT, "dladdr1");
   real_dlinfo = dlsym(RTLD_NEXT, "dlinfo");
 
-  libpulp_crash_assert(real_dlopen);
-  libpulp_crash_assert(real_dlmopen);
-  libpulp_crash_assert(real_dlclose);
-  libpulp_crash_assert(real_dladdr);
-  libpulp_crash_assert(real_dladdr1);
-  libpulp_crash_assert(real_dlinfo);
+  /* Check if we got the symbols we need from libdl.  */
+  if (!real_dlopen) {
+    set_libpulp_error_state_with_reason(ENOLIBDL, "unable to find function `dlopen`.");
+    ok = false;
+  }
+
+  if (!real_dlmopen) {
+    set_libpulp_error_state_with_reason(ENOLIBDL, "unable to find function `dlmopen`.");
+    ok = false;
+  }
+
+  if (!real_dlclose) {
+    set_libpulp_error_state_with_reason(ENOLIBDL, "unable to find function `dlclose`.");
+    ok = false;
+  }
+
+  if (!real_dladdr) {
+    set_libpulp_error_state_with_reason(ENOLIBDL, "unable to find function `dladdr`.");
+    ok = false;
+  }
+
+  if (!real_dladdr1) {
+    set_libpulp_error_state_with_reason(ENOLIBDL, "unable to find function `dladdr1`.");
+    ok = false;
+  }
+
+  if (!real_dlinfo) {
+    set_libpulp_error_state_with_reason(ENOLIBDL, "unable to find function `dlinfo`.");
+    ok = false;
+  }
 
   real_free = dlsym(RTLD_NEXT, "free");
   real_malloc = dlsym(RTLD_NEXT, "malloc");
   real_calloc = dlsym(RTLD_NEXT, "calloc");
   real_realloc = dlsym(RTLD_NEXT, "realloc");
-  real_reallocarray = dlsym(RTLD_NEXT, "reallocarray");
   real_valloc = dlsym(RTLD_NEXT, "valloc");
   real_pvalloc = dlsym(RTLD_NEXT, "pvalloc");
   real_memalign = dlsym(RTLD_NEXT, "memalign");
   real_aligned_alloc = dlsym(RTLD_NEXT, "aligned_alloc");
   real_posix_memalign = dlsym(RTLD_NEXT, "posix_memalign");
 
-  libpulp_crash_assert(real_free);
-  libpulp_crash_assert(real_malloc);
-  libpulp_crash_assert(real_calloc);
-  libpulp_crash_assert(real_realloc);
-  libpulp_crash_assert(real_reallocarray);
-  libpulp_crash_assert(real_valloc);
-  libpulp_crash_assert(real_pvalloc);
-  libpulp_crash_assert(real_memalign);
-  libpulp_crash_assert(real_aligned_alloc);
-  libpulp_crash_assert(real_posix_memalign);
-  libpulp_crash_assert(real_posix_memalign);
+  /* Check if we got the symbols we need from glibc.  */
+  if (!real_free) {
+    set_libpulp_error_state_with_reason(ENOLIBC, "unable to find function `free`.");
+    ok = false;
+  }
+
+  if (!real_malloc) {
+    set_libpulp_error_state_with_reason(ENOLIBC, "unable to find function `malloc`.");
+    ok = false;
+  }
+
+  if (!real_calloc) {
+    set_libpulp_error_state_with_reason(ENOLIBC, "unable to find function `calloc`.");
+    ok = false;
+  }
+
+  if (!real_realloc) {
+    set_libpulp_error_state_with_reason(ENOLIBC, "unable to find function `realloc`.");
+    ok = false;
+  }
+
+  if (!real_pvalloc) {
+    set_libpulp_error_state_with_reason(ENOLIBC, "unable to find function `pvalloc`.");
+    ok = false;
+  }
+
+  if (!real_memalign) {
+    set_libpulp_error_state_with_reason(ENOLIBC, "unable to find function `memalign`.");
+    ok = false;
+  }
+
+  if (!real_aligned_alloc) {
+    set_libpulp_error_state_with_reason(ENOLIBC, "unable to find function `aligned_alloc`.");
+    ok = false;
+  }
+
+  if (!real_posix_memalign) {
+    set_libpulp_error_state_with_reason(ENOLIBC, "unable to find function `posix_memalign`");
+    ok = false;
+  }
+
+  if (ok == false)
+    return;
 
   get_ld_global_locks();
 
@@ -729,8 +793,9 @@ realloc(void *ptr, size_t size)
 {
   void *result;
 
-  if (real_realloc == NULL)
-    __ulp_asunsafe_begin();
+  if (real_realloc == NULL) {
+    return __libc_realloc(ptr, size);
+  }
 
   __sync_fetch_and_add(&flag, 1);
   result = real_realloc(ptr, size);
@@ -739,28 +804,14 @@ realloc(void *ptr, size_t size)
   return result;
 }
 
-void *
-reallocarray(void *ptr, size_t nmemb, size_t size)
-{
-  void *result;
-
-  if (real_reallocarray == NULL)
-    __ulp_asunsafe_begin();
-
-  __sync_fetch_and_add(&flag, 1);
-  result = real_reallocarray(ptr, nmemb, size);
-  __sync_fetch_and_sub(&flag, 1);
-
-  return result;
-}
-
 void *
 valloc(size_t size)
 {
   void *result;
 
-  if (real_valloc == NULL)
-    __ulp_asunsafe_begin();
+  if (real_valloc == NULL) {
+    return __libc_valloc(size);
+  }
 
   __sync_fetch_and_add(&flag, 1);
   result = real_valloc(size);
@@ -774,8 +825,9 @@ pvalloc(size_t size)
 {
   void *result;
 
-  if (real_pvalloc == NULL)
-    __ulp_asunsafe_begin();
+  if (real_pvalloc == NULL) {
+    return __libc_pvalloc(size);
+  }
 
   __sync_fetch_and_add(&flag, 1);
   result = real_pvalloc(size);
@@ -789,8 +841,9 @@ memalign(size_t alignment, size_t size)
 {
   void *result;
 
-  if (real_memalign == NULL)
-    __ulp_asunsafe_begin();
+  if (real_memalign == NULL) {
+    return __libc_memalign(alignment, size);
+  }
 
   __sync_fetch_and_add(&flag, 1);
   result = real_memalign(alignment, size);
@@ -804,8 +857,15 @@ aligned_alloc(size_t alignment, size_t size)
 {
   void *result;
 
-  if (real_aligned_alloc == NULL)
-    __ulp_asunsafe_begin();
+  if (real_aligned_alloc == NULL) {
+    /* We have to emulate this function using memalign.  */
+    if ((alignment & (alignment - 1)) || (size & (alignment - 1))) {
+      errno = EINVAL;
+      return NULL;
+    }
+
+    return  __libc_memalign(alignment, size);
+  }
 
   __sync_fetch_and_add(&flag, 1);
   result = real_aligned_alloc(alignment, size);
@@ -819,8 +879,24 @@ posix_memalign(void **memptr, size_t alignment, size_t size)
 {
   int result;
 
-  if (real_posix_memalign == NULL)
-    __ulp_asunsafe_begin();
+  if (real_posix_memalign == NULL) {
+    void *mem;
+
+    /* Implement posix_memalign using memalign.  */
+    if (alignment % sizeof (void *) != 0
+        || !powerof2 (alignment / sizeof (void *))
+        || alignment == 0)
+      return EINVAL;
+
+    mem = __libc_memalign(alignment, size);
+
+    if (mem != NULL) {
+      *memptr = mem;
+      return 0;
+    }
+
+    return ENOMEM;
+  }
 
   __sync_fetch_and_add(&flag, 1);
   result = real_posix_memalign(memptr, alignment, size);
@@ -834,8 +910,9 @@ dlopen(const char *filename, int flags)
 {
   void *result;
 
-  if (real_dlopen == NULL)
+  if (real_dlopen == NULL) {
     __ulp_asunsafe_begin();
+  }
 
   __sync_fetch_and_add(&flag, 1);
   result = real_dlopen(filename, flags);
@@ -849,8 +926,9 @@ dlmopen(Lmid_t nsid, const char *file, int mode)
 {
   void *result;
 
-  if (real_dlmopen == NULL)
+  if (real_dlmopen == NULL) {
     __ulp_asunsafe_begin();
+  }
 
   __sync_fetch_and_add(&flag, 1);
   result = real_dlmopen(nsid, file, mode);
@@ -864,8 +942,9 @@ dlclose(void *handle)
 {
   int result;
 
-  if (real_dlclose == NULL)
+  if (real_dlclose == NULL) {
     __ulp_asunsafe_begin();
+  }
 
   __sync_fetch_and_add(&flag, 1);
   result = real_dlclose(handle);
@@ -879,8 +958,9 @@ dladdr(const void *address, Dl_info *info)
 {
   int result;
 
-  if (real_dladdr == NULL)
+  if (real_dladdr == NULL) {
     __ulp_asunsafe_begin();
+  }
 
   __sync_fetch_and_add(&flag, 1);
   result = real_dladdr(address, info);
@@ -894,8 +974,9 @@ dladdr1(const void *address, Dl_info *info, void **extra_info, int flags)
 {
   int result;
 
-  if (real_dladdr1 == NULL)
+  if (real_dladdr1 == NULL) {
     __ulp_asunsafe_begin();
+  }
 
   __sync_fetch_and_add(&flag, 1);
   result = real_dladdr1(address, info, extra_info, flags);
@@ -909,8 +990,9 @@ dlinfo(void *handle, int request, void *arg)
 {
   int result;
 
-  if (real_dlinfo == NULL)
+  if (real_dlinfo == NULL) {
     __ulp_asunsafe_begin();
+  }
 
   __sync_fetch_and_add(&flag, 1);
   result = real_dlinfo(handle, request, arg);
