@@ -50,6 +50,7 @@ struct arguments
   const char *process_wildcard;
   const char *user_wildcard;
   const char *prefix;
+  const char *with_debuginfo;
   command_t command;
   int retries;
   int quiet;
@@ -59,6 +60,8 @@ struct arguments
   int disable_threads;
   int recursive;
   int no_summarization;
+  int only_livepatched;
+  int disable_seccomp;
 #if defined ENABLE_STACK_CHECK && ENABLE_STACK_CHECK
   int check_stack;
 #endif
