@@ -526,7 +526,8 @@ parse_description(const char *filename, struct ulp_metadata *ulp,
   int cur_comment_size = 0;
 
   struct ulp_so_info *container = NULL, *target = NULL;
-  const char *container_path = NULL, *target_path = NULL;
+  const char *container_path = NULL;
+  char target_path[PATH_MAX];
 
   *info = NULL;
 
@@ -563,12 +564,14 @@ parse_description(const char *filename, struct ulp_metadata *ulp,
       goto dsc_clean;
     }
 
-    if (first[0] == '!') {
+    if (first[0] == '\n') {
+      /* Skip newline  */
+      continue;
+    } else if (first[0] == '!') {
       /* Lines starting with ! are comments.  */
       read_comment(ulp, first, n, &cur_comment_pos, &cur_comment_size);
       continue;
-    }
-    else {
+    } else {
       if (container_override) {
         ulp->so_filename = strdup(container_override);
       }
@@ -613,6 +616,15 @@ parse_description(const char *filename, struct ulp_metadata *ulp,
   container_path = ulp->so_filename;
   container = ulp_so_info_open(container_path);
 
+  if (container == NULL) {
+    /* Failed to open it.  The file is probably not what we expected it to be,
+       or has missing pieces.  */
+    parse_error(loc,
+          "Unsupported container format. Check if it is a valid ELF or JSON.");
+    ret = 0;
+    goto dsc_clean;
+  }
+
   n = getline(&first, &len, parse_file);
   loc.line++;
   loc.col = 1;
@@ -641,7 +653,10 @@ parse_description(const char *filename, struct ulp_metadata *ulp,
 
   while (n > 0) {
 
-    if (first[0] == '!') {
+    if (first[0] == '\n') {
+      /* Skip empty lines.  */
+      goto get_new_line;
+    } else if (first[0] == '!') {
       /* Lines starting with ! are comments.  */
       read_comment(ulp, first, n, &cur_comment_pos, &cur_comment_size);
     }
@@ -664,9 +679,9 @@ parse_description(const char *filename, struct ulp_metadata *ulp,
         first[n - 1] = '\0';
 
       if (target_override)
-        target_path = target_override;
+        strcpy(target_path, target_override);
       else
-        target_path = &first[1];
+        strcpy(target_path, &first[1]);
       ulp->objs->nunits = 0;
       last_unit = NULL;
 
@@ -685,6 +700,16 @@ parse_description(const char *filename, struct ulp_metadata *ulp,
       }
 
       target = ulp_so_info_open(target_path);
+
+      if (target == NULL) {
+        /* Failed to open it.  The file is probably not what we expected it to be,
+           or has missing pieces.  */
+        parse_error(loc,
+             "Unsupported target format. Check if it is a valid ELF or JSON.");
+        ret = 0;
+        goto dsc_clean;
+      }
+
       ulp->objs->name = strdup(target->name);
     }
     else {
@@ -731,21 +756,25 @@ parse_description(const char *filename, struct ulp_metadata *ulp,
         if (second == NULL) {
           ref->reference_name = strdup(third);
 
-          if (!get_symbol_with_name(target, ref->target_name)) {
+          struct symbol *new_sym = get_symbol_with_name(target, ref->target_name);
+          if (new_sym == NULL) {
             parse_error(loc, "symbol %s is not present in %s",
                         ref->target_name, target_path);
             ret = 0;
             goto dsc_clean;
           }
-
+          ref->target_offset = new_sym->offset;
           loc.col += strlen(ref->target_name) + 1;
 
-          if (!get_symbol_with_name(container, ref->reference_name)) {
+          struct symbol *old_sym = get_symbol_with_name(container, ref->reference_name);
+          if (old_sym == NULL) {
             parse_error(loc, "symbol %s is not present in %s",
                         ref->reference_name, container_path);
             ret = 0;
             goto dsc_clean;
           }
+
+          ref->patch_offset = old_sym->offset;
         }
         else {
           *second = '\0';
@@ -844,8 +873,8 @@ parse_description(const char *filename, struct ulp_metadata *ulp,
     }
 
     /* get new line */
-    free(first);
-    first = NULL;
+get_new_line:
+    FREE_AND_NULLIFY(first);
     second = NULL;
     len = 0;
     n = getline(&first, &len, parse_file);
@@ -948,33 +977,6 @@ parse_build_id(Elf_Scn *s, char **result, int *length)
   return 0;
 }
 
-void *
-get_symbol_addr(Elf *elf, Elf_Scn *s, const char *search)
-{
-  int nsyms, i;
-  char *sym_name;
-  Elf_Data *data;
-  GElf_Shdr sh;
-  GElf_Sym sym;
-
-  gelf_getshdr(s, &sh);
-
-  nsyms = sh.sh_size / sh.sh_entsize;
-  data = elf_getdata(s, NULL);
-  if (!data) {
-    WARN("Unable to get section data.");
-    return 0;
-  }
-
-  for (i = 0; i < nsyms; i++) {
-    gelf_getsym(data, i, &sym);
-    sym_name = elf_strptr(elf, sh.sh_link, sym.st_name);
-    if (strcmp(sym_name, search) == 0)
-      return (void *)sym.st_value;
-  }
-  return 0;
-}
-
 int
 write_patch_id(struct ulp_metadata *ulp, const char *description,
                const char *livepatch)
