@@ -394,35 +394,6 @@ print_lib_patches(struct ulp_applied_patch *patch, const char *libname)
   }
 }
 
-/** @brief Check if function at `sym_address` has the NOP preamble.
- *
- * Functions that are livepatchable has ULP_NOPS_LEN - PRE_NOPS_LEN at the
- * beginning of the function. Check the existence of this preamble.
- *
- * @param sym_address  Address of function in target process.
- * @param pid          Pid of the target process.
- *
- * @return  True if preamble exists, false if not.
- */
-static bool
-check_preamble(ElfW(Addr) sym_address, pid_t pid)
-{
-  unsigned char bytes[2];
-
-  if (read_memory((char *)bytes, 2, pid, sym_address)) {
-    /* In case it was unable to read the symbol due to permission error, just
-     * warn in debug output.  */
-    DEBUG("Unable to read symbol preamble at address %lx in process %d",
-          sym_address, pid);
-    return false;
-  }
-
-  /* Check for NOP NOP or XGCH AX, AX.  */
-  if ((bytes[0] == 0x90 || bytes[0] == 0x66) && bytes[1] == 0x90)
-    return true;
-  return false;
-}
-
 /** @brief Check if `libname` has a livepatch loaded.
  *
  * Check if the library with name `libname` has a livepatch loaded in the
@@ -453,6 +424,20 @@ has_livepatch_loaded(struct ulp_applied_patch *patch, const char *libname)
   return false;
 }
 
+/** @brief Check if function at `sym_address` has the NOP preamble.
+ *
+ * Functions that are livepatchable has ULP_NOPS_LEN - PRE_NOPS_LEN at the
+ * beginning of the function. Check the existence of this preamble.
+ *
+ * This function is arch specific.
+ *
+ * @param sym_address  Address of function in target process.
+ * @param pid          Pid of the target process.
+ *
+ * @return  True if preamble exists, false if not.
+ */
+bool check_preamble(ElfW(Addr) sym_address, pid_t pid);
+
 /** @brief Check if library in `obj` on target process is livepatchable.
  *
  * Check on the target process with `pid` if the library on `obj` is
@@ -466,14 +451,15 @@ has_livepatch_loaded(struct ulp_applied_patch *patch, const char *libname)
  *
  * @param patch  The list of patches loaded in the target process.
  * @param obj    The libary object.
- * @param pid    Pid of target process.
+ * @param p      The process to analyze.
  *
  * @return       True if livepatchable, False if not.
  */
 static bool
 is_library_livepatchable(struct ulp_applied_patch *patch,
-                         struct ulp_dynobj *obj, pid_t pid)
+                         struct ulp_dynobj *obj, struct ulp_process *p)
 {
+  pid_t pid = p->pid;
   int i, ret;
   if (has_livepatch_loaded(patch, obj->filename))
     return true;
@@ -484,15 +470,9 @@ is_library_livepatchable(struct ulp_applied_patch *patch,
     goto detach_process;
   }
 
-  ElfW(Addr) ehdr_addr = obj->link_map.l_addr;
+  ElfW(Addr) ehdr_addr = get_ehdr_addr(p, obj);
   ElfW(Addr) dynsym_addr = obj->dynsym_addr;
 
-  if (ehdr_addr == 0) {
-    /* If l_addr is zero, it means that there is no load bias.  In that case,
-     * the elf address is on address 0x400000 on x86_64.  */
-    ehdr_addr = 0x400000UL;
-  }
-
   /* FIXME: Some applications take a very long time to decide if library is
      livepatchable because the library has a lot of symbols.  In this case we
      limit the number of symbols to read to a constant value.  Statistics shows
@@ -576,7 +556,7 @@ print_process(struct ulp_process *process, int print_buildid,
   if (!object_item)
     printf("    (none)\n");
   while (object_item) {
-    if (is_library_livepatchable(patch, object_item, pid)) {
+    if (is_library_livepatchable(patch, object_item, process)) {
       printf("    in %s", object_item->filename);
       if (print_buildid)
         printf(" (%s)", buildid_to_string(object_item->build_id));
