@@ -130,6 +130,8 @@ static const char doc[] =
 #define ULP_OP_DISABLE_THREADING 260
 #define ULP_OP_RECURSIVE 261
 #define ULP_OP_DISABLE_SUMMARIZATION 262
+#define ULP_OP_ONLY_LIVEPATCHED 263
+#define ULP_OP_DISABLE_SECCOMP 264
 
 static struct argp_option options[] = {
   { 0, 0, 0, 0, "Options:", 0 },
@@ -142,6 +144,7 @@ static struct argp_option options[] = {
     "Do not launch additional threads", 0 },
   { 0, 0, 0, 0, "dump & patches command only:", 0 },
   { "buildid", 'b', 0, 0, "Print the build id", 0 },
+  { "only-livepatched", ULP_OP_ONLY_LIVEPATCHED, 0, 0, "Print only processes that were livepatched", 0 },
   { 0, 0, 0, 0, "trigger command only:", 0 },
   { "revert-all", ULP_OP_REVERT_ALL, "LIB", 0,
     "Revert all patches from LIB. If LIB=target, then all patches from the "
@@ -154,6 +157,8 @@ static struct argp_option options[] = {
   { "recursive", ULP_OP_RECURSIVE, 0, 0, "Search for patches recursively", 0 },
   { "root", 'R', "PREFIX", 0,
     "Append prefix to livepatch path when passing it to target process", 0 },
+  { "disable-seccomp", ULP_OP_DISABLE_SECCOMP, 0, 0,
+    "disable seccomp filters on target process (use for testing purposes)", 0 },
 #if defined ENABLE_STACK_CHECK && ENABLE_STACK_CHECK
   { "check-stack", 'c', 0, 0, "Check the call stack before live patching", 0 },
 #endif
@@ -169,8 +174,10 @@ static struct argp_option options[] = {
     "Use this livepatch file\nDefaults to the one described in ARG1", 0 },
   { "target", 't', "LIBRARY", 0,
     "Use this target library\nDefaults to the one described in ARG1", 0 },
-  { "color", ULP_OP_COLOR, "yes/no/auto", 0, "Enable/disable colored messages",
-    0 },
+  { "color", ULP_OP_COLOR, "yes/no/auto", 0, "Enable/disable colored messages", 0 },
+  { 0, 0, 0, 0, "extract command only:", 0 },
+  { "with-debuginfo", 'd', "DEBUGINFO", 0,
+    "Use debuginfo information for symbolextraction", 0 },
   { 0 }
 };
 
@@ -342,6 +349,9 @@ parser(int key, char *arg, struct argp_state *state)
     case 'R':
       arguments->prefix = arg;
       break;
+    case 'd':
+      arguments->with_debuginfo = arg;
+      break;
     case ULP_OP_REVERT_ALL:
       arguments->library = get_basename(arg);
       break;
@@ -360,6 +370,14 @@ parser(int key, char *arg, struct argp_state *state)
       arguments->no_summarization = 1;
       break;
 
+    case ULP_OP_ONLY_LIVEPATCHED:
+      arguments->only_livepatched = 1;
+      break;
+
+    case ULP_OP_DISABLE_SECCOMP:
+      arguments->disable_seccomp = 1;
+      break;
+
     case 'u':
       arguments->user_wildcard = arg;
       break;
@@ -444,6 +462,28 @@ change_color(const char *ansi_escape)
   }
 }
 
+static bool
+requires_ptrace(command_t command)
+{
+  switch(command) {
+    case ULP_NONE:
+    case ULP_DUMP:
+    case ULP_POST:
+    case ULP_EXTRACT:
+    case ULP_PACKER:
+    case ULP_LIVEPATCHABLE:
+      return false;
+
+    case ULP_PATCHES:
+    case ULP_TRIGGER:
+    case ULP_CHECK:
+    case ULP_MESSAGES:
+    case ULP_SET_PATCHABLE:
+      return true;
+  }
+  return false;
+}
+
 int
 main(int argc, char **argv, char *envp[] __attribute__((unused)))
 {
@@ -470,6 +510,17 @@ main(int argc, char **argv, char *envp[] __attribute__((unused)))
 
   argp_parse(&argp, argc, argv, 0, 0, &arguments);
 
+
+  /* Check if command requires ptrace.  */
+  if (requires_ptrace(arguments.command) &&
+        check_ptrace_scope() == false) {
+    WARN("System has 'ptrace_scope' enabled. Please become root or disable it "
+         "by setting:\n\n"
+         "$ echo 0 | sudo tee /proc/sys/kernel/yama/ptrace_scope\n\n"
+         "and try again.");
+    return EPERM;
+  }
+
   switch (arguments.command) {
     case ULP_NONE:
       ret = 1;
